#include "drv_w25qxx.h"
#include "sspi.h"
#include "spi_demo.h"
#include "string.h"
#include "delay_demo.h"
#define SPI_POLLING
//#define SPI_WITH_DMA

uint16_t drv_w25qxx_read_device_id(void)
{
    uint16_t tmp = 0;
    static uint8_t rxbuf[6];
    static uint8_t txbuf[7] = {0x90, 0x00, 0x00, 0x00, 0xff, 0xff};
#ifdef  SPI_POLLING
    SSPI_Start(SSPI21);

    SSPI_TransmitData(SSPI21,0x90);
    SSPI_TransmitData(SSPI21,0x00);
    SSPI_TransmitData(SSPI21,0x00);
    SSPI_TransmitData(SSPI21,0x00);
    tmp |= SSPI_TransmitData(SSPI21,0xFF) << 8;
    tmp |= SSPI_TransmitData(SSPI21,0xFF);

    SSPI_Stop(SSPI21);

    return tmp;
#else
#ifdef SPI_WITH_DMA
    Spi11_Dma_TxRx(txbuf, 6, rxbuf, 6);
    delayMS(4);
    while (g_spi_tx_end == 0);

    tmp |= (rxbuf[4] << 8);
    tmp |= rxbuf[5];
    return tmp;
#else
    SSPI_Start(SSPI11);
	
    SSPI_Set_TransmitMode(SSPI11, SSPI_TransmitMode_TxRx);

    SSPI_TransmitData(SSPI11,0x90);
    SSPI_TransmitData(SSPI11,0x00);
    SSPI_TransmitData(SSPI11,0x00);
    SSPI_TransmitData(SSPI11,0x00);

    Spi11_Int_Read(rxbuf, 2);
    delayMS(1);
    while (g_spi_rx_end == 0);

    tmp |= (rxbuf[0] << 8);
    tmp |= rxbuf[1];

    return tmp;
#endif
#endif
}

uint32_t drv_w25qxx_read_jedec_id(void)
{
    uint32_t tmp = 0;
    static uint8_t rxbuf[5];
    static uint8_t txbuf[5] = {0x9F, 0xff, 0xff, 0xff, 0xff};
#ifdef  SPI_POLLING
    SSPI_Start(SSPI11);
	
    SSPI_TransmitData(SSPI11,0x9F);
    tmp |=  SSPI_TransmitData(SSPI11,0xFF) << 16;
    tmp |=  SSPI_TransmitData(SSPI11,0xFF) << 8;
    tmp |=  SSPI_TransmitData(SSPI11,0xFF);

    SSPI_Stop(SSPI11);

    return tmp;
#else
#ifdef SPI_WITH_DMA
    Spi11_Dma_TxRx(txbuf, 5, rxbuf, 5);

    delayMS(1);
    while (g_spi_tx_end == 0);

    tmp |= (rxbuf[1] << 16);
    tmp |= (rxbuf[2] << 8);
    tmp |= rxbuf[3];
    return tmp;
#else
    SSPI_Start(SSPI11);
	
    SSPI_Set_TransmitMode(SSPI11, SSPI_TransmitMode_TxRx);
	SSPI_TransmitData(SSPI11,0x9F);

    Spi11_Int_Read(rxbuf, sizeof(rxbuf));
    delayMS(1);	
    while (g_spi_rx_end == 0);

    tmp |= (rxbuf[0] << 16);
    tmp |= (rxbuf[1] << 8);
    tmp |= rxbuf[2];
    return tmp;
#endif
#endif
}

void drv_w25qxx_read_uniq_id(uint8_t *buf)
{
    static uint8_t txbuf[13] = {0x4B, 0xff, 0xff, 0xff, 0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
    static uint8_t rxbuf[13];
#ifdef  SPI_POLLING
	SSPI_Start(SSPI11);

	SSPI_TransmitData(SSPI11,0x4B);

    for (uint8_t i = 0; i < 4; i++)
    {
		SSPI_TransmitData(SSPI11,0x00);
    }

    for (uint8_t i = 0; i < 8; i++)
    {
        buf[i] = SSPI_TransmitData(SSPI11,0xFF);
    }

    SSPI_Stop(SSPI11);
#else
#ifdef SPI_WITH_DMA
    Spi11_Dma_TxRx(txbuf, 13, rxbuf, 13);

    delayMS(2);
    while (g_spi_tx_end == 0);

    memcpy(buf, &rxbuf[5], 8);
#else
    SSPI_Start(SSPI11);
	
    SSPI_Set_TransmitMode(SSPI11, SSPI_TransmitMode_TxRx);
	SSPI_TransmitData(SSPI11,0x4B);

    for (uint8_t i = 0; i < 4; i++)
    {
		SSPI_TransmitData(SSPI11,0x00);
    }

    Spi11_Int_Read(buf, 8);

    delayMS(1);
    while (g_spi_rx_end == 0);
#endif
#endif
}
