/*******************************************************************************
*                  COPYRIGHT (C) 2021 CMS Technologies Ltd.                    *
*                                                                              *
********************************************************************************
* FileName      : tim_demo.c                                                  *
* Author        :                                                              *
* Version       : 1.0                                                          *
* Date          : 2021.08.13                                                   *
* Description   :                                                              *
* Function List :                                                              *
********************************************************************************/
#include "tima_demo.h"
#include "gpio_demo.h"
#include "uart_demo.h"

volatile uint16_t g_tma0_underflow_count = 0U;
volatile uint16_t g_tma0_ta0_value = 0xffff;   /* TA0 = 65535 */


/***********************************************************************************************************************
* Function Name: TMA_SquareOutput
* @brief  The TMA operates as square output.
* @param  clk - select count source clock
* @param  num - specify the count value
* @return None
***********************************************************************************************************************/
void TMA_SquareOutput(TMA_Clk_t clk, uint16_t num)
{
	GPIO_InitTypeDef GPIO_InitStruct={0};	
	TMA_InitTypeDef TIMA_InitStructure={0};

	GPIO_PinAFConfig(GPIO_PORT0,GPIO_Pin_1,GPIO_P01,GROUP_AF_ODEFAULT);  //P01 default used as TAIO output

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_1;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_OUT;	
	GPIO_InitStruct.GPIO_Level	= GPIO_Level_LOW;
	GPIO_InitStruct.GPIO_Ctrl	= GPIO_Control_DIG;
	GPIO_Init(GPIO_PORT0,&GPIO_InitStruct);
	
	TIMA_InitStructure.TMA_Clk = clk;   //specify the operation clk of tim
	TIMA_InitStructure.TMA_Period = num;                 //specify the number of count clock 
	TIMA_InitStructure.TMA_Mode = TMA_Mode_Square;      // Square mode
	TIMA_InitStructure.TMA_Polarity = TMA_Polarity_0;  //init level:(TIMA_Polarity_0/high level)(TIMA_Polarity_1/low level)

	TMA_Init(&TIMA_InitStructure);

    TMA_Start();
}

/***********************************************************************************************************************
* Function Name: TMA_EventCount
* @brief  The TMA operates as event counter.
* @param  edge - specify the valid edge and count condition
* @param  num - specify the count value
* @return None
***********************************************************************************************************************/
void TMA_EventCount_Init(uint16_t tma_edge, uint16_t tma_ctrol,uint16_t num)
{
	GPIO_InitTypeDef GPIO_InitStruct={0};	
	TMA_InitTypeDef TIMA_InitStructure={0};

	GPIO_PinAFConfig(GPIO_PORT0,GPIO_Pin_1,GPIO_P01,GROUP_AF_ODEFAULT);  //P01 default used as TAIO input event

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_1;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_IN;	
	GPIO_InitStruct.GPIO_Ctrl	= GPIO_Control_DIG;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;	
	GPIO_Init(GPIO_PORT0,&GPIO_InitStruct);

	TIMA_InitStructure.TMA_Clk = TMA_FCLK_Div1;         //specify the operation clk of tim
	TIMA_InitStructure.TMA_Period  = num;                 //specify the number of count clock 
	TIMA_InitStructure.TMA_Mode    = TMA_Mode_EventCount;      // EventCount mode
	TIMA_InitStructure.TMA_EventPara.TIMA_Edge = tma_edge; //ؽ¼
	TIMA_InitStructure.TMA_EventPara.TMA_Ctrl  = tma_ctrol;//INTP4Ŵڸߵƽ¼

	TMA_Init(&TIMA_InitStructure);

	ISR_Register(TMA_IRQn, tima_interrupt);  //жϷ·ע
}

/***********************************************************************************************************************
* Function Name: TMA_IntervalTimer
* @brief This function interval timer and generate interval interrupt.
* @note   Interval timer period calculation 1/clk *TIM_Period = timerValue (s)
* @return none
***********************************************************************************************************************/
void TMA_IntervalTimer(TMA_Clk_t clk, uint16_t num)
{
	TMA_InitTypeDef TIMA_InitStructure={0};
	
	TIMA_InitStructure.TMA_Clk = clk;   //specify the operation clk of tim
	TIMA_InitStructure.TMA_Period = num;                 //specify the number of count clock 
	TIMA_InitStructure.TMA_Mode = TMA_Mode_Interval;      // timer mode

	TMA_Init(&TIMA_InitStructure);
	ISR_Register(TMA_IRQn, tima_timerinterrupt);  //жϷ·ע
}


/***********************************************************************************************************************
* Function Name: TMA_Get_PulseWidth
* @brief  The TMA operates as pulse width measure.
* @param  clk - select count source clock
* @param  level - specify the measure level
* @return none
***********************************************************************************************************************/
void TMA_Get_PulseWidth(uint16_t clk, TMA_Pulse_t level)
{
	uint32_t width;
	GPIO_InitTypeDef GPIO_InitStruct={0};	
	TMA_InitTypeDef TIMA_InitStructure = {0};

	GPIO_PinAFConfig(GPIO_PORT0,GPIO_Pin_1,GPIO_P01,GROUP_AF_ODEFAULT);  //P01 used as TAIO input

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_1;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_IN;	
	GPIO_InitStruct.GPIO_Ctrl	= GPIO_Control_DIG;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;	
	GPIO_Init(GPIO_PORT0,&GPIO_InitStruct);

	TIMA_InitStructure.TMA_Clk = clk;   //specify the operation clk of tim
	TIMA_InitStructure.TMA_Period  = g_tma0_ta0_value;       //specify the number of count clock 
	TIMA_InitStructure.TMA_Mode    = TMA_Mode_PluseWidth;      // EventCount mode
	TIMA_InitStructure.TMA_Pulse   = level;
	TMA_Init(&TIMA_InitStructure);
}

/***********************************************************************************************************************
* Function Name: TMA0_Get_PulsePeriod
* @brief  The TMA operates as pulse width measure.
* @param  clk - select count source clock
* @param  edge - specify the rising edge or falling edge
* @return none
***********************************************************************************************************************/
void TMA_Get_PulsePeriod(uint16_t clk, TMA_Pulse_t edge)
{
	uint32_t width;
	volatile uint16_t tmp;
	GPIO_InitTypeDef GPIO_InitStruct = {0};	
	TMA_InitTypeDef TIMA_InitStructure = {0};

	GPIO_PinAFConfig(GPIO_PORT0,GPIO_Pin_1,GPIO_P01,GROUP_AF_ODEFAULT);  //P00 used as TAIO input

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_1;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_IN;	
	GPIO_InitStruct.GPIO_Ctrl	= GPIO_Control_DIG;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;	
	GPIO_Init(GPIO_PORT0,&GPIO_InitStruct);

	TIMA_InitStructure.TMA_Clk = clk;   //specify the operation clk of tim
	TIMA_InitStructure.TMA_Period  = g_tma0_ta0_value;       //specify the number of count clock 
	TIMA_InitStructure.TMA_Mode    = TMA_Mode_PlusePeriod;      // EventCount mode
	TIMA_InitStructure.TMA_Pulse   = edge;
	TMA_Init(&TIMA_InitStructure);
	
}

void tima_interrupt()
{
    INTC_ClearPendingIRQ(TMA_IRQn); /* clear INTTMA interrupt flag */
	
    if (TMA_GetStaus(TMA_FLAG_UNDERFLOW))
    {
		TMA_ClearFlag(TMA_FLAG_UNDERFLOW);
        g_tma0_underflow_count += 1U;
    }	
}

void tima_timerinterrupt()
{
    INTC_ClearPendingIRQ(TMA_IRQn); /* clear INTTMA interrupt flag */
	
    if (TMA_GetStaus(TMA_FLAG_UNDERFLOW))
    {
		TMA_ClearFlag(TMA_FLAG_UNDERFLOW);
		GPIO_ToggleBits(GPIO_PORT3,GPIO_Pin_1);
    }	
}