/*******************************************************************************
*                  COPYRIGHT (C) 2021 CMS Technologies Ltd.                    *
*                                                                              *
********************************************************************************
* FileName      : tim_demo.c                                                  *
* Author        :                                                              *
* Version       : 1.0                                                          *
* Date          : 2021.08.13                                                   *
* Description   :                                                              *
* Function List :                                                              *
********************************************************************************/
#include "timb_demo.h"
#include "gpio_demo.h"
#include "uart_demo.h"

volatile uint16_t g_tmb_intTaken_a = 0U;
volatile uint16_t g_tmb_intTaken_b = 0U;
volatile uint16_t g_tmb_overflow_count_a = 0U;
volatile uint16_t g_tmb_overflow_count_b = 0U;
volatile uint32_t g_tmb_active_width_a = 0UL;
volatile uint32_t g_tmb_active_width_b = 0UL;

/***********************************************************************************************************************
* Function Name: TMB_IntervalTimer
* @brief This function interval timer and generate interval interrupt.
* @note   Interval timer period calculation 1/clk *TMB_CHA_Pulse = timerValue (s)
* @return none
***********************************************************************************************************************/
void TMB_IntervalTimer(TMB_Clk_t clk, uint16_t num)
{
	TMB_InitTypeDef TIMB_InitStructure={};

	TIMB_InitStructure.TMB_Channel = TMB_Channel_A;
		
	TIMB_InitStructure.TMB_Clk = clk;   //specify the operation clk of tim
	TIMB_InitStructure.TMB_CHA_Pulse = num;
	TIMB_InitStructure.TMB_Mode = TMB_Mode_Compare;      // timer mode
	TIMB_InitStructure.TMB_CounterClear = TB_Clear_Match_TBGRA;  //when TB is matched with TBGRB, TB is cleared

	TMB_Init(&TIMB_InitStructure);
	ISR_Register(TMB_IRQn, timb_timerinterrupt);  //жϷ·ע
}


/***********************************************************************************************************************
* Function Name: TMB_DoublePulseOutput
* @brief  The TMB operates as double pulse output.
* @param  clk - select count source clock
* @param  period - the period of TBIO0 and TBIO1 output square wave.
* @param  skew - the phase skew from the risging edge of TBIO0 to the rising edge of TBIO1.
* @return None
* @note   The value of skew MUST be less than the value of half period. [gap < (period / 2)]
*
*            ^
*       FFFFH|
*            |
* TBGRB = m-1|_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
*            |                    _| |                    _| |                    _| |
*            |                  _|   |                  _|   |                  _|   |
*            |                _|     |                _|     |                _|     |
*            |              _|       |              _|       |              _|       |        
*            |            _|         |            _|         |            _|         |
* TMGRA = n-1|_ _ _ _ _ _|_ _ _ _ _ _|_ _ _ _ _ _|_ _ _ _ _ _|_ _ _ _ _ _|_ _ _ _ _ _|_ _ _ _ 
*            |        _| |           |        _| |           |        _| |           |         
*            |      _|   |           |      _|   |           |      _|   |           |       _  
*            |    _|     |           |    _|     |           |    _|     |           |     _|    
*            |  _|       |           |  _|       |           |  _|       |           |   _|      
*       0000H|_|_________|___________|_|_________|___________|_|_________|___________|__|______
*                        |           |           |           |           |           |
*                        |___________|___________|           |           |___________|_________
*     TBIO0 _____________|           |           |___________|___________|           |         
*                        |____       |           |____       |           |____       |   
* TBSR.IMFA _____________|    |______|___________|    |______|___________|    |______|_________
*                                    |                       |                       |          
*                                    |_______________________|                       |_________
*     TBIO1 _________________________|                       |_______________________|         
*                                    |____                   |____                   |____
* TBSR.IMFB _________________________|    |__________________|    |__________________|    |____
*                                                 
***********************************************************************************************************************/
void TMB_DoublePulseOutput(uint8_t clk, uint16_t period, uint16_t gap)
{	
	GPIO_InitTypeDef GPIO_InitStruct;	
	TMB_InitTypeDef TIMB_InitStructure;

	GPIO_PinAFConfig(GPIO_PORT5,GPIO_Pin_0,GPIO_P50,GROUP_AF_ODEFAULT);  //P50 used as TBO0 output(default function)
	GPIO_PinAFConfig(GPIO_PORT5,GPIO_Pin_1,GPIO_P51,GROUP_AF_ODEFAULT);  //P51 used as TBO1 output(default function)

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_0 | GPIO_Pin_1;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_OUT;	
	GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;	
	GPIO_InitStruct.GPIO_OType	= GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;		
	GPIO_Init(GPIO_PORT5,&GPIO_InitStruct);
	
	TIMB_InitStructure.TMB_Channel = TMB_Channel_A|TMB_Channel_B;
	TIMB_InitStructure.TMB_Clk = clk;                       //specify the operation clk of tim
	TIMB_InitStructure.TMB_Mode    = TMB_Mode_Compare;                       // Compare mode
	
	TIMB_InitStructure.TMB_CHA_Pulse = period/2 - gap;
	TIMB_InitStructure.TMB_OC.TMB_CHA.TMB_OCPolarity = TMB_Output_Toggle;   //TBﵽTBGRA,ƽзת

	TIMB_InitStructure.TMB_CHB_Pulse = period/2;					//TBﵽTBGRB,ƽзת
	TIMB_InitStructure.TMB_OC.TMB_CHB.TMB_OCPolarity = TMB_Output_Toggle;	 //
	
	TIMB_InitStructure.TMB_CounterClear = TB_Clear_Match_TBGRB;  //when TB is matched with TBGRB, TB is cleared
	
	TMB_Init(&TIMB_InitStructure);


//	ISR_Register(TMB_IRQn, tmb_interrupt);  //жϷ·ע
	
	TMB_Start();
}


/***********************************************************************************************************************
* Function Name: TMB0_PWM_1Period_1Duty
* @brief  The TMB operates as PWM output mode.
* @param  clk - select count source clock
* @param  period - the period of TBIO0 output wave
* @param  duty - the duty of TBIO0 output wave
* @return None
*
*            ^
*       FFFFH|
*            |
* TBGRA = m-1|_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
*            |                    _| |                    _| |                    _| |
*            |                  _|   |                  _|   |                  _|   |
*            |                _|     |                _|     |                _|     |
*            |              _|       |              _|       |              _|       |        
*            |            _|         |            _|         |            _|         |
* TMGRB = n-1|_ _ _ _ _ _|_ _ _ _ _ _|_ _ _ _ _ _|_ _ _ _ _ _|_ _ _ _ _ _|_ _ _ _ _ _|_ _ _ _ 
*            |        _| |           |        _| |           |        _| |           |         
*            |      _|   |           |      _|   |           |      _|   |           |       _  
*            |    _|     |           |    _|     |           |    _|     |           |     _|    
*            |  _|       |           |  _|       |           |  _|       |           |   _|      
*       0000H|_|_________|___________|_|_________|___________|_|_________|___________|__|______
*                        |           |           |           |           |           |
*           _____________|           |___________|           |___________|           |_________
*     TBIO0              |___________|           |___________|           |___________|         
*                        |____       |           |____       |           |____       |   
* TBSR.IMFB _____________|    |______|___________|    |______|___________|    |______|_________
*                                    |____                   |____                   |____      
* TBSR.IMFA _________________________|    |__________________|    |__________________|    |____
*                                                 
***********************************************************************************************************************/
void TMB_PWM_1Period_1Duty(uint8_t clk, uint16_t period, uint16_t duty)
{
	GPIO_InitTypeDef GPIO_InitStruct;	
	TMB_InitTypeDef TIMB_InitStructure ={};

	GPIO_PinAFConfig(GPIO_PORT5,GPIO_Pin_0,GPIO_P50,GROUP_AF_ODEFAULT);  //P50 used as TBIO0(TMB_Channel_A) output(default function)

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_0;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;	
	GPIO_InitStruct.GPIO_OType	= GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;	
	GPIO_Init(GPIO_PORT5,&GPIO_InitStruct);

	TIMB_InitStructure.TMB_Channel = TMB_Channel_A;  		//NOTE: output PWM channel must be TMB_Channel_A,
	TIMB_InitStructure.TMB_Clk = clk;                       //specify the operation clk of tim
	TIMB_InitStructure.TMB_Mode    = TMB_Mode_PWM;          // PWM mode	
	TIMB_InitStructure.TMB_CHA_Pulse = period;
	TIMB_InitStructure.TMB_CHB_Pulse = duty;				
	
	TIMB_InitStructure.TMB_CounterClear = TB_Clear_Match_TBGRA;  //when TB is matched with TBGRA, TB is cleared
	
	TMB_Init(&TIMB_InitStructure);
	
	TMB_Start();
}

/***********************************************************************************************************************
* Function Name: TMB_Get_PulseWidth
* @brief  The TMB operates as pulse width measure.
* @param  clk - select count source clock
* @param  level - specify the measure level
* @return none
***********************************************************************************************************************/
void TMB_Get_PulseWidth_Init(uint8_t clk, uint8_t capture_edge)
{
	uint32_t width;
	GPIO_InitTypeDef GPIO_InitStruct;	
	TMB_InitTypeDef TIMB_InitStructure;


	GPIO_PinAFConfig(GPIO_PORT5,GPIO_Pin_0,GPIO_P50,GROUP_AF_ODEFAULT);  //P01 used as TBIO0 input

	GPIO_InitStruct.GPIO_Pin	= GPIO_Pin_0;	
	GPIO_InitStruct.GPIO_Mode	= GPIO_Mode_IN;	
	GPIO_InitStruct.GPIO_Ctrl	= GPIO_Control_DIG;
	GPIO_InitStruct.GPIO_PuPd	= GPIO_PuPd_NOPULL;	
	GPIO_Init(GPIO_PORT5,&GPIO_InitStruct);

	TIMB_InitStructure.TMB_Channel = TMB_Channel_A;
	TIMB_InitStructure.TMB_Clk = clk;   //specify the operation clk of tim
	TIMB_InitStructure.TMB_Mode    = TMB_Mode_Capture;      // Capture mode
	TIMB_InitStructure.TMB_IC.TMB_CHA.TMB_ICPolarity = capture_edge;//
	
	TMB_Init(&TIMB_InitStructure);
}




void timb_timerinterrupt()
{
    INTC_ClearPendingIRQ(TMB_IRQn); /* clear INTTMB interrupt flag */
    uint8_t tbsr_temp = TMB->TBSR;
    uint8_t tbier_temp = TMB->TBIER;

    TMB->TBIER = 0x00U;

    if(TMB_GetStaus(TMB_FLAG_CHA_MATCHED) == SET)
    {
        TMB->TBSR = tbsr_temp & ~TMB_FLAG_CHA_MATCHED;
		GPIO_ToggleBits(GPIO_PORT3,GPIO_Pin_1);
    }

    TMB->TBIER = tbier_temp;
}

void tmb_interrupt()
{
    INTC_ClearPendingIRQ(TMB_IRQn); /* clear INTTMB interrupt flag */
    uint8_t tbsr_temp = TMB->TBSR;
    uint8_t tbier_temp = TMB->TBIER;

    TMB->TBIER = 0x00U;

	if(TMB_GetStaus(TMB_FLAG_TB_OVERFLOW) == SET)
    {
        TMB->TBSR = tbsr_temp & (uint8_t)~TMB_FLAG_TB_OVERFLOW;
        g_tmb_overflow_count_a += 1UL;
        g_tmb_overflow_count_b += 1UL;
    }

#if 1
    if(TMB_GetStaus(TMB_FLAG_CHA_MATCHED) == SET)
    {
        TMB->TBSR = tbsr_temp & ~TMB_FLAG_CHA_MATCHED;
        g_tmb_intTaken_a++;

        if (g_tmb_overflow_count_a == 0UL)
        {
            g_tmb_active_width_a = (TMB->TBGRA - TMB->TBGRC);
        }
        else
        {
            g_tmb_active_width_a = (TMB->TBGRA - TMB->TBGRC) + (0x10000UL * g_tmb_overflow_count_a);
            g_tmb_overflow_count_a = 0UL;
        }
    }

    if (TMB_GetStaus(TMB_FLAG_CHB_MATCHED) == SET)
    {
        TMB->TBSR = tbsr_temp & ~TMB_FLAG_CHB_MATCHED;
        g_tmb_intTaken_b++;

        if (g_tmb_overflow_count_b == 0UL)
        {
            g_tmb_active_width_b = (TMB->TBGRB - TMB->TBGRD);
        }
        else
        {
            g_tmb_active_width_b = (TMB->TBGRB - TMB->TBGRD) + (0x10000UL * g_tmb_overflow_count_b);
            g_tmb_overflow_count_b = 0UL;
        }
    }
#endif
    TMB->TBIER = tbier_temp;
}

