/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32A239
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/4/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "gpio_demo.h"
#include "uart_demo.h"
#include "timm_demo.h"



/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/

int main()
{
	uint32_t msCnt; 	// count value of 1ms

//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------   
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt); 

//-----------------------------------------------------------------------
// Ouptput Compare function
// P17/TMIOA0 output pulse wave
// P14/TMIOB0 output pulse wave
//-----------------------------------------------------------------------
/*            ^
 *       FFFFH|
 *            |
 *TMGRA0 = m-1|_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
 *            |                    _| |                    _| |
 *            |                  _|   |                  _|   |
 *            |                _|     |                _|     |
 *TMGRC0 = n-1|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _
 *            |            _| |       |            _| |       |
 *TMGRB0 = p-1|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _
 *            |        _| |   |       |        _| |   |       |        
 *            |      _|   |   |       |      _|   |   |       |      _  
 *TMGRD0 = q-1|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|    
 *            |  _| |     |   |       |  _| |     |   |       |  _|      
 *       0000H|_|___|_____|___|_______|_|___|_____|___|_______|_|______
 *                  |     |   |       |     |     |   |       |
 *                  |     |   |_______|     |     |   |_______|
 *     TMIOA0 ______|_____|___|       |_____|_____|___|       |________
 *                  |     |   |       |__   |     |   |       |__
 * TMSR0.IMFA ______|_____|___|_______|  |__|_____|___|_______|  |_____
 *                  |     |   |__________   |     |   |__________
 * TMSR0.IMFC ______|_____|___|          |__|_____|___|          |_____
 *                  |     |                 |     |
 *                  |_____|                 |_____|
 *     TMIOB0 ______|     |_________________|     |____________________
 *                  |     |______________   |     |______________
 * TMSR0.IMFB ______|_____|              |__|_____|              |_____
 *                  |____________________   |____________________
 * TMSR0.IMFD ______|                    |__|                    |_____
 *                                                 
 */
//	TMM0_DoublePulseOutput(TMM_FCLK_Div1, 1000,800,600,200);


//-----------------------------------------------------------------------
// PWM output function
// P14/TMIOB0 output PWM wave (duty: 80%)
// P16/TMIOC0 output PWM wave (duty: 50%) 
// P15/TMIOD0 output PWM wave (duty: 20%)
//-----------------------------------------------------------------------
/*            ^
 *       FFFFH|
 *            |
 *TMGRA0 = m-1|_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
 *            |                    _| |                    _| |
 *            |                  _|   |                  _|   |
 *            |                _|     |                _|     |
 *TMGRB0 = n-1|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _
 *            |            _| |       |            _| |       |
 *TMGRC0 = p-1|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _
 *            |        _| |   |       |        _| |   |       |
 *            |      _|   |   |       |      _|   |   |       |      _
 *TMGRD0 = q-1|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|
 *            |  _| |     |   |       |  _| |     |   |       |  _|
 *       0000H|_|___|_____|___|_______|_|___|_____|___|_______|_|______
 *                  |     |   |       |     |     |   |       |
 *            ______|_____|___|       |_____|_____|___|       |________
 *     TMIOB0       |     |   |_______|     |     |   |_______|
 *            ______|_____|   |       |_____|_____|   |       |________
 *     TMIOC0       |     |___|_______|     |     |___|_______|
 *            ______|     |   |       |_____|     |   |       |________
 *     TMIOD0       |_____|___|_______|     |_____|___|_______|
 *                  |     |   |       |     |     |   |       |
 *                  |     |   |       |__   |     |   |       |__
 * TMSR0.IMFA ______|_____|___|_______|  |__|_____|___|_______|  |_____
 *                  |     |   |__________   |     |   |__________
 * TMSR0.IMFB ______|_____|___|          |__|_____|___|          |_____
 *                  |     |______________   |     |______________
 * TMSR0.IMFC ______|_____|              |__|_____|              |_____
 *                  |____________________   |____________________
 * TMSR0.IMFD ______|                    |__|                    |_____
 *
 */
	TMM0_PWM_1Period_3Duty(TMM_FCLK_Div1,1000, 800, 500, 200);


/***********************************************************************************************************************
//Function Name: TMM0_RstSynchronous_PWM
// @brief  output 6 PWM with same period and reverse phase PWM .
// P14/TMIOB0 output PWM wave (duty: 80%)
// P15/TMIOD0 output PWM wave (duty: 20%)
// P10/TMIOB0 output PWM wave (duty: 60%)
// P11/TMIOD0 output PWM wave (duty: 40%)
// P12/TMIOB0 output PWM wave (duty: 30%)
// P13/TMIOD0 output PWM wave (duty: 70%)
*            ^
*       FFFFH|
*            |
*TMGRA0 = m-1|_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
*            |                    _| |                    _| |
*            |                  _|   |                  _|   |
*            |                _|     |                _|     |
*TMGRB0 = n-1|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _ _ _ _ _|_ _ _ _|_ _ _ _
*            |            _| |       |            _| |       |
*TMGRA1 = p-1|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _ _ _|_ _|_ _ _ _|_ _ _ _
*            |        _| |   |       |        _| |   |       |        
*            |      _|   |   |       |      _|   |   |       |      _  
*TMGRB1 = q-1|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|_ _ _|_ _|_ _ _ _|_ _ _|    
*            |  _| |     |   |       |  _| |     |   |       |  _|      
*       0000H|_|___|_____|___|_______|_|___|_____|___|_______|_|______
*                  |     |   |       |     |     |   |       |
*            ______|_____|___|       |_____|_____|___|       |________
*     TMIOB0       |     |   |_______|     |     |   |_______|        
*                  |     |   |       |     |     |   |       |
*                             _______|     |     |   |_______
*     TMIOD0 ______|_____|___|       |_____|_____|___|       |________        
*                  |     |   |       |     |     |   |       |  
*                  |     |   |       |     |     |   |       |
*            ______|     |   |       |_____|     |   |       |________
*     TMIOB1       |_____|___|_______|     |_____|___|_______|        
*                  |     |   |       |     |     |   |       |
*                   _____|___|_______|     |_____|___|_______|
*     TMIOD1 ______|     |   |       |_____|     |   |       |________        
*                  |     |   |       |     |     |   |       |  
*                  |     |   |       |     |     |   |       |
*            ______|_____|   |       |_____|_____|   |       |________
*     TMIOA1       |     |___|_______|     |     |___|_______|        
*                  |     |   |       |     |     |   |       |
*                        |___|_______|     |     |___|_______|
*     TMIOC1 ______|_____|   |       |_____|_____|   |       |_____        
*                  |     |   |       |     |     |   |       |  
*                  |     |   |       |     |     |   |       |                                                
**/
//	TMM0_RstSynchronous_PWM(TMM_FCLK_Div1, 1000, 800, 600, 300);


}




